#
# $Id: grid.py 244 2006-04-01 20:43:16Z dave $ 
#
import time, unittest
import machine

def DEBUG():
    return True

class GridError(Exception):
    pass

class GridUsedTile(GridError):
    pass
        
class GridComponent:
    isPipe = False
    def __init__(self, component, size, position=None, layout=0):
        self.component = component #used only to get the real slots
        self.position = position
        self.size = size
        self.layoutn = layout
        self.makeTiles()
        self.sizes=[self.size]

    def reset(self):
        if self.component<>None:
            self.component.reset()
            
    def __str__(self):
        if self.position<>None:
            return self.__class__.__name__+":"+str(self.getSlots())
        return self.__class__.__name__+":unused yet"

    def makeTiles(self):
        sx,sy = self.size
        tiles = []
        for tx in range(sx):
            for ty in range(sy):
                tiles.append( (tx, ty) )
        self._tiles = tiles
        
    def tiles(self, position=None):
        if position<>None:
            ax,ay=position
        else:
            ax,ay=self.position
        return map (lambda (x,y):(ax+x,ay+y), self._tiles)      

    def hasSlotIn(self, tiles):
        #if self.isPipe:
        #    #don't check the pipes
        #    return None
        for name, pos in self.getSlots():
            if pos in tiles:
                try:
                    return name #self.component[name]
                except:
                    if DEBUG(): print self.component
                    raise
        return None

    def slotToAbsolute(self, (name, slot)):
        """give a slot(which are relative) is converted to absolute position"""
        px, py = self.position
        return name, (slot[0]+px, slot[1]+py)

    def getSlots(self):
        return map(self.slotToAbsolute, self._slots[self.layoutn].items())
    def getSlotPos(self, name):
        return self.slotToAbsolute((name , self._slots[self.layoutn][name]))
    
    def getSlotTilePos(self, slotName):
        """horrible.."""
        return self.slotToAbsolute( (slotName, self._slots[self.layoutn][slotName]) )
        
    def getSlotRelativePos(self, slotName):
        x, y = self._slots[self.layoutn][slotName]
        sx,sy= self.size
        pos = [ ]
        if y<0:
            pos.append('top')
        elif y>=sy:
            pos.append('bottom')
        if x<0:
            pos.append('left')
        elif x>=sx:
            pos.append('right') 
        return pos
    
    def rotate(self):
        self.layoutn+=1
        self.layoutn%=len(self._slots)
        self.size=self.sizes[self.layoutn%len(self.sizes)]
        self.makeTiles()

    def isBroken(self):
        if self.isPipe: return False
        return self.component.broken

class NullGridComponent(GridComponent):
    """
        Size : (size, 1)
        Slots: @ (-1,1), (size.x+1, 1)
        
         pipe are special components, 'cause doesn't have really components
        and have size!! 
    """
    isPipe = True
    def __init__(self, position=None):
        GridComponent.__init__(self, None, (1,1), position)
        self._slots = []
    def getSlots(self):
        return []
    def rotate(self):
        pass

    
class HorizontalPipeGridComponent(GridComponent):
    """
        Size : (size, 1)
        Slots: @ (-1,1), (size.x+1, 1)
        
         pipe are special components, 'cause doesn't have really components
        and have size!! 
    """
    isPipe = True    
    def __init__(self, _size, position=None):
        _size = (_size, 1)
        GridComponent.__init__(self, None, _size, position)
        sx = _size[0]
        self._slots = [{'a':(-1,0), 'b':(sx,0)}]

    def makeTiles(self):
        sx,sy = self.size
        self._tiles = [(0, 0), (sx-1, 0) ]

    def invisibleTile(self, pos):
        tx,ty = pos
        sx,sy = self.size
        px,py = self.position
        return (ty==py) and (px<=tx) and (tx<px+sx)
        
class VerticalPipeGridComponent(GridComponent):
    """
        Size : (1, size)
        Slots: @ (-1,1), (size.x+1, 1)
        
         pipe are special components, 'cause doesn't have really components
        and have size!! 
    """
    isPipe = True    
    def __init__(self, _size, position=None):
        _size = (1, _size)
        GridComponent.__init__(self, None, _size, position)
        sx = _size[1]
        self._slots = [{'a':(0,-1), 'b':(0,sx)}]
    def makeTiles(self):
        sx,sy = self.size
        self._tiles = [(0, 0), (0, sy-1) ]
    def invisibleTile(self, pos):
        tx,ty = pos
        sx,sy = self.size
        px,py = self.position
        return (tx==px) and (py<=ty) and (ty<py+sy)

class HorizontalBeltGridComponent(HorizontalPipeGridComponent): 
    pass
    
class VerticalBeltGridComponent(VerticalPipeGridComponent):
    pass

        
class BoilerGridComponent(GridComponent):
    """
        Size : 4x3
        Slots: @ (9,2)
        BBBB
        BBBB*
        BBBB
    """
    size = (8,3)
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.BoilerComponent(), (4,3), position)
        sx,sy=self.size
        self._slots = [{'output':(sx,1)}]

class GaugeGridComponent(GridComponent):
    """
        Size :  2x2
        Slots: @ (-1,2), (3,2)
         GG
        iGGo 
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.GaugeComponent(), (2,2), position)
        sx,sy=self.size
        self._slots = [{'input':(-1,sy-1), 'output':(sx,sy-1)}]
    
    def getValue(self):
        return self.component.value


class TGridComponent(GridComponent):
    """
        Size :  3x2
        Slots: @ (-1,1), (4,1), (2,3)
        aTTTc
         tTt
          b
          
         Rotates: XXX
         2)    3)     4)
         c     b      a
         T     T      T
         TTb cTTTa  bTT
         T            T
         a            c
    """
    def __init__(self, position=None, layout=0):
        GridComponent.__init__(self, machine.TComponent(), (3,2), position, layout)
        sx,sy=self.size
        self._slots = [{'A':(-1,0), 'B':(1,sy), 'C':(sx,0)},    
            {'A':(0,sx), 'B':(sy,+1), 'C':(0,-1) }, #2
            {'A':(sx,1),  'B':(+1,-1), 'C':(-1,sy-1)}, #3
            {'A':(sy-1,-1), 'B':(-1,+1), 'C':(sy-1,sx)} ] #4
        self.sizes = [ (3,2), (2,3) ]
        
class XGridComponent(GridComponent):
    """
        Size :  3x3
        Slots: @ (-1,2), (2,-1), (2,3), (3,2)
          *
         tTt
        *TTT*
         tTt
          *
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.XComponent(), (3,3), position)
        sx, sy = self.size
        self._slots = [{'A':(-1,1),'B':(1,-1), 'C':(sx,1), 'D':(1,sy) }]

class ElbowGridComponent(GridComponent):
    """
        Size :  2x2
        Slots: @ ...
          *   *EE     EE*   *               
         EE    EE     EE    EE 
        *EE     *     *     EE*      
        Rotates: XXX
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.ElbowComponent(), (2,2), position)
        sx,sy=self.size
        self._slots = [
            {'A':(sx-1,-1), 'B':(-1,sy-1)},
            {'A':(-1,0), 'B':(sx-1,sy)},
            {'A':(0,sy), 'B':(sx,0)},
            {'A':(sx,sy-1), 'B':(0,-1)}]

class ElbowGearGridComponent(GridComponent):
    """
        Size :  2x2
        Slots: @ ...
          *   *EE     EE*   *               
         EE    EE     EE    EE 
        *EE     *     *     EE*      
        Rotates: XXX
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.ElbowGearComponent(), (2,2), position)
        sx,sy=self.size
        self._slots = [
            {'A':(sx-1,-1), 'B':(-1,sy-1)},
            {'A':(-1,0), 'B':(sx-1,sy)},
            {'A':(0,sy), 'B':(sx,0)},
            {'A':(0,-1), 'B':(sx,sy-1)}]


class ManualGearGridComponent(GridComponent):
    """
        Size :  3x3
        Slots: @ (1,2)
          *
         GGG
         GGG
         GGG
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.ManualGearComponent(), (3,3), position)
        sx,sy=self.size
        self._slots = [{'gear':(1,-1)}]

    def setValue(self, value):
        self.component.setValue(value)
        
    def getValue(self):
        return self.component.getValue()
    value = property(getValue, setValue)


class GearValveGridComponent(GridComponent):
    """
        Size :  3x2
          *
         GGG
        *GGG*
          *
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.GearValveComponent(), (3,2), position)
        sx,sy=self.size
        self._slots = [{'A':(-1,1),'B':(sx,1),'escape':(1,-1),'gear':(1,sy)}]

class PressureToGearGridComponent(GridComponent):
    """
        Size :  1x1
          *
         GGG
        *GGG*
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, 
            machine.PressureToGearComponent(), (3,2), position)
        sx,sy=self.size
        self._slots = [{'input':(-1,1),'output':(sx,1),'gear':(1,-1)}]

class TankGridComponent(GridComponent):
    """
        Size :  3x3
          *
         TTT 
         TTT 
        *TTT
          *
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.TankComponent(), (3,3), position)
        sx,sy=self.size
        self._slots = [{'input':(-1,sy-1), 'output':(1,-1),'reset':(1,sy)}]

class SmallTankGridComponent(GridComponent):
    """
        Size :  3x2
          *
         TTT 
        *TTT
          *
    """
    def __init__(self, position=None):
        GridComponent.__init__(self, machine.SmallTankComponent(), (3,2), position)
        sx,sy=self.size
        self._slots = [{'input':(-1,sy-1), 'output':(1,-1), 'reset':(1,sy)}]

class MachineGridComponent(GridComponent):
    """ Component is ignored here.. """
    def __init__(self, component, size, position=None, slotDict={}): #, tiles=[]):
        GridComponent.__init__(self, component, (0,0), position)
        self._slots = [slotDict]
        self.msize = size
        self._tiles = []
        for y in range(self.msize[1]):
            self._tiles += [ (0,y), (self.msize[0]-1,y) ]
        #self._tiles+=tiles
    
    def tiles(self, position=None):
        """ we have a fixed position """
        return self._tiles      

class _RawGrid:
    """usar diccionario?"""
    def __init__(self, size):
        self.width, self.height = size
        x,y=size
        self.grid=[]
        for ty in range(y):
            l=[None]*x
            self.grid.append(l)
        self.slots = {}
        #slots contains a dictionary with this form:
        #    { (x,y):[(component1,'slotname'),(component2,'slotname')],...}
            
    def dump(self, slots=False, dontPrint=False):
        output = ''
        maxx = 30

        if not dontPrint:
            linecount='+'
            for i in range(maxx):
                linecount+=str(i)[len(str(i))-1]
            print 
            print '-'*(maxx+1)
            print linecount
            l = []
            for y in range(10):
                line = ' '*maxx
                l.append(line)

        #slots contains a dictionary with this form:
        #    { (x,y):[(component1,'slotname'),(component2,'slotname')],...}

        for y in range(10):
            s=''
            for x in range(maxx):
                t=' '
                comp = self.getComponentAt((x,y))
                if comp<>None:
                    t=comp.__class__.__name__[0]
                elif slots:
                    try:
                        c = len(self.slots[(x,y)])
                        if (c>0):
                            if c>=10: c=9
                            t=str(c)
                    except: pass
                s+=t
            output+=s+'\n'
            if not dontPrint: print str(y)[len(str(y))-1]+s
        if not dontPrint: print linecount
        return output.replace(' ','')
        
    def isEmpty(self, x, y):
        return self.getComponentAt((x,y))==None
    
    def outsideGridBounds(self, position):
        x,y = position
        return x<0 or y<0 or x>=self.width or y>=self.height

    def checkTilesAvailableFor(self, component, position=None):
        for tile in component.tiles(position):
            if self.outsideGridBounds(tile):
                return False
            x,y = tile
            if not self.grid[y][x] in [None, component]:
                return False
        return True

    def _resetTiles(self, comp):
        self._setTiles(comp, True)
    
    def _setTiles(self, comp, resetTiles=False):
        what = comp
        if resetTiles:
            what=None
        for tile in comp.tiles():
            x,y = tile
            self.grid[y][x]=what
                
    def add(self, comp):
        if not self.checkTilesAvailableFor(comp):
            raise GridUsedTile()
        self._setTiles(comp)
        for name, pos in comp.getSlots():
            slist = self.slots.setdefault(pos,[])
            slist.append( (comp, name) ) 
            
    def remove(self, comp):
        self._resetTiles(comp)
        for name, pos in comp.getSlots():
            #se puede mejorar..
            slist = self.slots[pos]
            slist.remove( (comp, name) )
    
    def move(self, comp):
        self.remove(comp)
        self.add(comp)
            
    def getComponentSlotsAt(self, pos):
        """returns the list of components/slots at a given position"""
        return self.slots.setdefault(pos,[])    
    
    def getComponentAt(self, pos):
        """None if there's no component"""
        if self.outsideGridBounds(pos):
            return None
        x,y=pos
        return self.grid[y][x]
    
class Grid:
    def dump(self, *args, **kwargs):
        return self.grid.dump(*args, **kwargs)

    def __init__(self, _size, machineFactory, slots={}, nulls=[]):
        """
            Receives: 
                machineFactory: a function that creates machine objects
                slots: a dictionary of slots tiles {'name':(posx, posy)..}
                nulls: a list of null (non-moveable) tiles
         """
        global DEBUG
        self.machineFactory = machineFactory
        self.grid = _RawGrid(_size)
        self.startTime = time.time()
        self.components = {}
        self.log = []
        
        #make grid slots!
        if DEBUG(): print 'machine slots:',slots
        self.machinecomp = MachineGridComponent (None, _size, (0,0), slots) #, nulls)
        self.grid.add(self.machinecomp)
        for pos in nulls:
            try:
                self.grid.add(NullGridComponent(pos)) 
            except GridUsedTile:
                #it's ok to ignore them
                pass

    def getState(self):
        pass #return [self.components, self.links, self.grid]
    def setState(self, state):
        pass #self.componentes, self.conexiones, self.grid = state

    def _addLog(self, log):
        self.log.append( (time.time()-self.startTime, log) )
        
    def tryComponent(self, component, position=None):
        return self.grid.checkTilesAvailableFor(component, position)
        
    def addComponent(self, component, position=None):
        if position<>None: component.position = position
        self.grid.add(component)
        self.components[component]=None
        self._addLog(['addComponent', component]) 
    
    def removeComponent(self, component):
        comp = self.components[component]
        del self.components[component]
        self.grid.remove(component)
        self._addLog(['removeComponent', component])

    def rotateComponent(self, component):
        self.removeComponent(component)
        component.rotate()
        while not self.tryComponent(component):
            #at least original position should work!
            component.rotate()
        self.addComponent(component)
        self._addLog(['rotateComponent', component])

    def getComponentAt(self, pos, checkPhantasm=False):
        """CheckPhantasm: check also if pos is a tile from inside a pipe"""
        comp = self.grid.getComponentAt(pos)
        if comp==None and checkPhantasm:
            for pipeComp in self.components:
                if pipeComp.isPipe:
                    if pipeComp.invisibleTile(pos):
                        return pipeComp
        return comp
        
    def reset(self):
        self.machine.reset()

    def start(self):
        global DEBUG
        """Makes the machine and starts it"""
        self.machine = self.machineFactory()
        self.machinecomp.component = self.machine
        
        components, bindings = self.makeBindings()
        for c in components:
            c.reset()
            self.machine.addComponent(c.component)
        for bind in bindings:
            try:
                self.machine.connect(bind[0],bind[1])
            except:
                if DEBUG():
                    self.grid.dump(True)
        
    def makeBindings(self):
        components = []
        bindings = []
        pipes = []
        used = []

        for c in self.components.keys():
            if not c.isPipe:
                components.append(c)
            else:
                pipes.append(c)

        #check bindings
        for pipe in pipes:
            if DEBUG(): print 'Revisando pipe:',pipe 
            #solo revisamos pipes con al menos un slot en un componente, 
            #para minimizar cantidad de chequeos..
            slots = pipe.getSlots()
            ends = map(lambda pos:self.getComponentAt(pos[1]), slots)
            if DEBUG(): print 'Ends:',ends
            if ends[0]==None or ends[1]==None:
                break
            if ends[0].isPipe and ends[1].isPipe:
                break

            compA = self.getEndToSlot(pipe, slots[0][0])
            compB = self.getEndToSlot(pipe, slots[1][0])
            if compA<>None and compB<>None:
                a,nameA = compA
                b,nameB = compB
                if DEBUG(): print 'Conexion (',a, nameA,' <-> ', b, nameB,')'
                slotA = a.component[nameA]
                slotB = b.component[nameB]
                if (not slotA in used) and (not slotB in used):
                    bindings.append( (slotA, slotB))
                    used.append(slotA)
                    used.append(slotB)

                
        #connect component between themself 
        if DEBUG(): print 'connection direct components!!!!'
        for comp1 in self.components.keys():
            if not comp1.isPipe:
                for slotname, pos in comp1.getSlots():
                    if not (comp1.component[slotname] in used):
                        component = self.getComponentAt(pos)
                        if component<>None:
                            #slot = component.hasSlotIn(comp1.tiles())
                            #if slot<>None and not (slot in used):
                            name = component.hasSlotIn(comp1.tiles())
                            if name<>None and not component.isPipe:
                                slot = component.component[name]
                                if not (slot in used):
                                    try:
                                        other = comp1.component[slotname]
                                        bindings.append((other,slot))
                                        used.append(other)
                                        if DEBUG(): print 'Ok'
                                    except machine.ConnectionError, e:
                                        if not e.args[0].startswith('Slot already connected:'):
                                            raise
        return components, bindings


    def getEndToSlot(self, component, slotName):
        name, slotPos = component.getSlotPos(slotName)
        end=self.getComponentAt(slotPos)
        if end<>None:
            name = self.getSlotFromTo(end, component)
            if name:
                if end.isPipe:
                    l = ['a','b']
                    l.remove(name) 
                    return self.getEndToSlot(end, l[0])
                else:
                    return end, name
                    
    def getSlotFromTo(self, fromComp, toComp):
        return fromComp.hasSlotIn(toComp.tiles())
                            
    
    def step(self, steps=1):
        self.machine.step(steps)


def machineFactory():
    m = machine.Machine({"MIL":machine.PipeSlot})
    m.setValidator(lambda s: s["MIL"].in_value == 1000)
    return m


class GridTests(unittest.TestCase):    
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, 
                                    { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testOverlay(self):
        grid = self.grid
        b = BoilerGridComponent((1,1))
        grid.addComponent( b )  
        b = BoilerGridComponent((8,3))
        self.assertRaises(GridError, grid.addComponent, b )
        b = BoilerGridComponent((9,2))
        grid.addComponent( b )

    def testOverlay2(self):
        grid = self.grid
        b = BoilerGridComponent()
        b.position = (1,1)
        grid.addComponent( b )
        b = BoilerGridComponent()
        self.assertEqual(grid.tryComponent(b,(8,3)),False)
        b.position = (8,3)
        self.assertRaises(GridError, grid.addComponent, b )
        b = BoilerGridComponent((9,3))
        grid.addComponent( b )
        
    def testMachineConnection(self):
        grid = self.grid
        b = BoilerGridComponent((1,1))
        grid.addComponent( b )
        #grid.dump(True)
        
        h = HorizontalPipeGridComponent( 10, (9,2))
        grid.addComponent( h )
        #grid.dump(True)
        
        g = GaugeGridComponent((19, 1))
        grid.addComponent( g )
        #grid.dump(True)

        g = TGridComponent((21, 5))
        grid.addComponent( g )
        #g.move((21,1))
        #grid.dump()

        h1 = HorizontalPipeGridComponent(10)
        grid.addComponent(h1,(21,1))
        #grid.dump()
        
        grid.start()
        grid.step(100)
        #grid.dump(True)    

class TestGridLayoutsTests(unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testGaugeGrid(self):
        grid = self.grid
        e = GaugeGridComponent()
        e.position=(1,0)
        grid.addComponent(e)
        #grid.dump(True)

    def testTGrid(self):
        grid = self.grid
        e = TGridComponent()
        e.position=(1,0)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),
            '1TTT1\nTTT\n1\n\n\n\n\n\n\n\n')
    
    def testXGrid(self):
        grid = self.grid
        e = XGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'1\nXXX\n1XXX1\nXXX\n1\n\n\n\n\n\n')

    def testElbowGrid(self):
        grid = self.grid
        e = ElbowGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'1\n1E\n\n\n\n\n\n\n\n\n')

    def testElbowGrid(self):
        grid = self.grid
        e = ElbowGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'1\n1E\n\n\n\n\n\n\n\n\n')

    def testMGGrid(self):
        grid = self.grid
        e = ManualGearGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'M\n1\n\n\n\n\n\n\n\n\n')

    def testMGGrid(self):
        grid = self.grid
        e = ManualGearGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'M\n1\n\n\n\n\n\n\n\n\n')
    
    def testGVGrid(self):
        grid = self.grid
        e = GearValveGridComponent()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'1\n1G1\n1\n\n\n\n\n\n\n\n')

    def testPressureToGearGridComponent(self):
        grid = self.grid
        e = PressureToGearGridComponent ()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'1\n1P1\n\n\n\n\n\n\n\n\n')

    def testTankGridComponent(self):
        grid = self.grid
        e = TankGridComponent ()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'\nTTT\n1TTT1\n1\n\n\n\n\n\n\n')

    def testSmallTankGridComponent(self):
        grid = self.grid
        e = SmallTankGridComponent ()
        e.position=(1,1)
        grid.addComponent(e)
        self.assertEqual(
            grid.dump(True,True),'\n1SSS1\n1\n\n\n\n\n\n\n\n')

class NullGridTests (unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, 
            { 'reset':(35,0), 'MIL':(35,1)}, [(3,3)] ) 

    def testNullTileWorks(self):
        grid = self.grid        
        b = TGridComponent((10,10))
        b.position=(3,3)
        self.assertRaises(GridUsedTile, self.grid.addComponent, b)


class RotateTGridTests (unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testBLAOverlay(self):
        grid = self.grid        
        b = TGridComponent((10,10))
        b.position=(3,3)
        grid.addComponent(b)
        grid.dump(True)
        #print b.size
        grid.rotateComponent(b)
        grid.dump(True)
        #print b.size
        grid.rotateComponent(b)
        grid.dump(True)
        #print b.size
        grid.rotateComponent(b)
        grid.dump(True)
        #print b.size

    def testElbowOverlay(self):
        grid = self.grid        
        b = ElbowGridComponent((10,10))
        b.position=(3,3)
        grid.addComponent(b)
        grid.dump(True)
        grid.rotateComponent(b)
        grid.dump(True)
        grid.rotateComponent(b)
        grid.dump(True)
        grid.rotateComponent(b)
        grid.dump(True)
        
    def testlucio(self):
        grid = self.grid        
        b = HorizontalPipeGridComponent(6,(1,1))
        grid.addComponent(b)
        grid.dump(True)
        b = VerticalPipeGridComponent(5,(6,2))
        grid.addComponent(b)
        grid.dump(True)

_COMPONENT_LIST = [
    (machine.GaugeComponent, GaugeGridComponent),
    (machine.BoilerComponent, BoilerGridComponent),
    (machine.ElbowComponent, ElbowGridComponent),
    (machine.ElbowGearComponent, ElbowGearGridComponent),
    (machine.TComponent, TGridComponent),
    (machine.XComponent, XGridComponent),
    (machine.ManualGearComponent, ManualGearGridComponent),
    (machine.GearValveComponent, GearValveGridComponent),
    (machine.PressureToGearComponent, PressureToGearGridComponent),
    (machine.TankComponent, TankGridComponent),
    (machine.SmallTankComponent, SmallTankGridComponent),
]

class ComponentSlotNamesTests (unittest.TestCase):
    def testSlotNameMatches(self):
        global _COMPONENT_LIST
        for comp,gcomp in _COMPONENT_LIST:
            a = comp.slots.keys()
            a.sort()
            b = map(lambda (x,y):x, gcomp((0,0)).getSlots() )
            b.sort()
            self.assertEqual(a,b)

import levels

class SlotPositionTests (unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testElbowOverlay(self):
        grid = self.grid        
        b = ElbowGridComponent((10,10))
        b.position=(3,3)
        grid.addComponent(b)
        for t in range(4):
            #grid.dump(True)
            print 'A:',b.getSlotRelativePos('A'),b.getSlotTilePos('A')
            print 'B:',b.getSlotRelativePos('B'),b.getSlotTilePos('B')
            grid.rotateComponent(b)

class BindingTests (unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testSimplePipeBindings(self):
        grid = self.grid
        b = GaugeGridComponent((2,2))
        grid.addComponent(b)
        
        b2 = HorizontalPipeGridComponent(6,(4,3))
        grid.addComponent(b2)

        b = GaugeGridComponent((10,2))
        grid.addComponent(b)
        grid.dump(True)
                
        comp, binds = grid.makeBindings()
        print 'Componentes:',comp
        print 'Conexiones:',binds

    def testBindCompAgainstComp(self):
        grid = self.grid
        b = GaugeGridComponent((2,2))
        grid.addComponent(b)
        
        b = GaugeGridComponent((4,2))
        grid.addComponent(b)
        grid.dump(True)
        
        comp, binds = grid.makeBindings()
        print 'Componentes:',comp
        print 'Conexiones:',binds

    def testDualPipeBindings(self):
        grid = self.grid
        b = GaugeGridComponent((2,2))
        grid.addComponent(b)
        
        b2 = HorizontalPipeGridComponent(6,(4,3))
        grid.addComponent(b2)
        b3 = HorizontalPipeGridComponent(6,(10,3))
        grid.addComponent(b3)

        b = GaugeGridComponent((16,2))
        grid.addComponent(b)
        grid.dump(True)
        
        comp, binds = grid.makeBindings()
        print 'Componentes:',comp
        print 'Conexiones:',binds

class GearLinkTests (unittest.TestCase):
    def setUp(self):        
        self.grid = Grid((35,35), machineFactory, { 'reset':(35,0), 'MIL':(35,1)} ) 

    def testManualGearLayout(self):
        grid = self.grid
        b = ManualGearGridComponent((2,2))
        grid.addComponent(b)
        grid.dump(True)
    def testGearValveLayout(self):
        grid = self.grid
        b = GearValveGridComponent((2,2))
        grid.addComponent(b)
        grid.dump(True)
    def testPressureToGearLayout(self):
        grid = self.grid
        b = PressureToGearGridComponent((2,2))
        grid.addComponent(b)
        grid.dump(True)

if __name__ == "__main__":
    suite = unittest.makeSuite(SlotPositionTests)
    suiteBindings = unittest.makeSuite(BindingTests)
    suiteGear = unittest.makeSuite(GearLinkTests)
    alltests = unittest.TestSuite((suiteGear))
    #alltests = unittest.TestSuite((suite, suiteBindings, suiteGear))
    
    unittest.TextTestRunner(verbosity=2222).run(alltests)
    if 0:
        suite = unittest.makeSuite(GridTests)
        #suite2 = unittest.makeSuite(RotateTGridTests)
        #suite3 = unittest.makeSuite(TestGridLayoutsTests)
        suite4 = unittest.makeSuite(NullGridTests)
        suite5 = unittest.makeSuite(ComponentSlotNamesTests)
        alltests = unittest.TestSuite((suite,suite4, suite5))
        unittest.TextTestRunner(verbosity=2222).run(alltests)
    
