"""animation loading and manipulating functions.

please note that this file is alpha, and is subject to modification in
future versions of pgu!"""

import pygame

def _ani_load(tv,name,parts,frames,shape):
    l = len(frames)
    #print name,parts,l
    n = parts.pop()
    if len(parts):
        s = l/n
        for i in xrange(0,n):
            _ani_load(tv,name + ".%d"%i,parts[:],frames[s*i:s*(i+1)],shape)
        return
    
    for i in xrange(0,n):
        tv.images[name+".%d"%i] = frames[i],shape

def ani_load(tv,name,img,size,shape,parts):
    """load an animation from an image
    
    tv -- vid to load into
    name -- prefix name to give the images
    size -- w,h size of image
    shape -- shape of image (usually a subset of 0,0,w,h) used for collision detection
    parts -- list of parts to divide the animation into 
        for example parts = [4,5] would yield 4 animations 5 frames long, 20 total
        for example parts = [a,b,c] would yield ... images['name.a.b.c'] ..., a*b*c total
    
    """
    parts = parts[:]
    parts.reverse()
    w,h = size
    frames = []
    for y in xrange(0,img.get_height(),h):
        for x in xrange(0,img.get_width(),w):
            frames.append(img.subsurface(x,y,w,h))
    _ani_load(tv,name,parts,frames,shape)
    
    
def image_rotate(tv,name,img,shape,angles,diff=0):
    """rotate an image and put it into tv.images
    
    tv -- vid to load into
    name -- prefix name to give the images
    shape -- shape fimage (usually a subset of 0,0,w,h) used for collision detection, not rotated
    angles -- a list of angles to render in degrees
    diff -- a number to add to the angles, to correct for source image not actually being at 0 degrees
    """
    shape = pygame.Rect(shape)
    w,h = shape.w,shape.h
    for a in angles:
        img2 = pygame.transform.rotate(img,a+diff)
        ww,hh = img2.get_width(),img2.get_height()
        tv.images["%s.%d"%(name,a)] = img2,((ww-w)/2,(hh-h)/2,w,h)

