import pygame
import widget
from pygame.locals import *
from base import *
from functions import *

_SLIDER_HORIZONTAL = 0
_SLIDER_VERTICAL = 1

class _slider(widget.Widget):
    def __init__(self,value,orient,min,max,size,**params):
        params.setdefault('cls','slider')
        widget.Widget.__init__(self,**params)
        self.min,self.max,self.value,self.orient,self.size = min,max,value,orient,size
    
    def paint(self,s):
        self.pcls = ""
        if self.container.myhover is self: self.pcls = "hover"
        
        r = pygame.rect.Rect(0,0,self.rect.w,self.rect.h)
        #render_box(s,self.style.background,r)
        if self.orient == _SLIDER_HORIZONTAL:
            r.x = (self.value-self.min) * (self.rect.w-self.size) / (self.max-self.min);
            r.w = self.size;
        else:
            r.y = (self.value-self.min) * (self.rect.h-self.size) / (self.max-self.min);
            r.h = self.size;
        render_box(s,self.style.bar,r)
    
    def event(self,e):
        adj = 0
        if e.type == ENTER: self.repaint()
        elif e.type == EXIT: self.repaint()
        elif e.type == KEYDOWN:
            if e.key == K_TAB:
                self.next()
        elif e.type == MOUSEBUTTONDOWN: 
            x,y,adj = e.pos[0],e.pos[1],1
        elif e.type == MOUSEBUTTONUP:
            x,y,adj = e.pos[0],e.pos[1],1
        elif e.type == MOUSEMOTION:
            if 1 in e.buttons and self.container.myfocus is self:
                x,y,adj = e.pos[0],e.pos[1],1
        if adj:
            if self.orient == _SLIDER_HORIZONTAL:
                d = self.size/2 - (self.rect.w/(self.max-self.min+1))/2
                self.value = (x-d) * (self.max-self.min) / (self.rect.w-self.size+1) + self.min
            else:
                d = self.size/2 - (self.rect.h/(self.max-self.min+1))/2
                self.value = (y-d) * (self.max-self.min) / (self.rect.h-self.size+1) + self.min
    
    def __setattr__(self,k,v):
        if k == 'value':
            v = int(v)
            v = max(v,self.min)
            v = min(v,self.max)
        _v = self.__dict__.get(k,NOATTR)
        self.__dict__[k]=v
        if k == 'value' and _v != NOATTR and _v != v: 
            self.send(CHANGE)
            self.repaint()

class VSlider(_slider):
    def __init__(self,value,min,max,size,**params):
        params.setdefault('cls','vslider')
        _slider.__init__(self,value,_SLIDER_VERTICAL,min,max,size,**params)

class HSlider(_slider):
    def __init__(self,value,min,max,size,**params):
        params.setdefault('cls','hslider')
        _slider.__init__(self,value,_SLIDER_HORIZONTAL,min,max,size,**params)
	
class VScrollBar(_slider):
    def __init__(self,value,min,max,size,**params):
        params.setdefault('cls','vscrollbar')
        _slider.__init__(self,value,_SLIDER_VERTICAL,min,max,size,**params)

class HScrollBar(_slider):
    def __init__(self,value,min,max,size,**params):
        params.setdefault('cls','hscrollbar')
        _slider.__init__(self,value,_SLIDER_HORIZONTAL,min,max,size,**params)
