import os, re
import pygame
from base import *
from functions import *

def _list_themes(dir):
    d = {}
    for entry in os.listdir(dir):
        if os.path.exists(os.path.join(dir, entry, 'config.txt')):
            d[entry] = os.path.join(dir, entry)
    return d

class Theme:
    """theme engine
    
    pgu.gui.Theme("theme")
    pgu.gui.Theme(["theme1","theme2"])
    """
    def __init__(self,ds='default'):
        self.config = {}
        self.dict = {}
        self._loaded = []
        self.cache = {}
        self._preload(ds)
    
    def _preload(self,ds):
        if not isinstance(ds, list):
            ds = [ds]
        for d in ds:
            if d not in self._loaded:
                self._load(d)
            self._loaded.append(d)
    
    def _load(self, name):
        #theme_dir = themes[name]
        
        #try to load the local dir, or absolute path
        dnames = [name]
        
        #if the package isn't installed and people are just
        #trying out the scripts or examples
        dnames.append(os.path.join(os.path.dirname(__file__),"..","..","data","themes",name))
        
        #if the package is installed, and the package is installed
        #in /usr/lib/python2.3/site-packages/pgu/
        #or c:\python23\lib\site-packages\pgu\
        #the data is in ... lib/../share/ ...
        dnames.append(os.path.join(os.path.dirname(__file__),"..","..","..","..","share","pgu","themes",name))
        dnames.append(os.path.join(os.path.dirname(__file__),"..","..","..","..","..","share","pgu","themes",name))
        
        for dname in dnames:
            if os.path.isdir(dname): break
        if not os.path.isdir(dname): 
            raise 'could not find theme '+name
            
        fname = os.path.join(dname, "config.txt")
        try:
            f = open(fname)
            for line in f.readlines():
                vals = line.strip().split()
                if len(vals) < 3: continue
                cls = vals[0]
                del vals[0]
                pcls = ""
                if cls.find(":")>=0:
                    cls,pcls = cls.split(":")
                attr = vals[0]
                del vals[0]
                self.config[cls+":"+pcls+" "+attr] = (dname, vals)
        finally:
            f.close()
    
    is_image = re.compile('\.(gif|jpg|bmp|png|tga)$', re.I)
    def _get(self,key):
        if not key in self.config: return
        if key in self.dict: return self.dict[key]
        dvals = self.config[key]
        dname, vals = dvals
        #theme_dir = themes[name]
        v0 = vals[0]
        if v0[0] == '#':
            v = pygame.color.Color(v0)
        elif v0.endswith(".ttf") or v0.endswith(".TTF"):
            v = pygame.font.Font(os.path.join(dname, v0),int(vals[1]))
        elif self.is_image.search(v0) is not None:
            v = pygame.image.load(os.path.join(dname, v0))
        else:
            try: v = int(v0)
            except: v = pygame.font.SysFont(v0, int(vals[1]))
        self.dict[key] = v
        return v    
    
    def get(self,cls,pcls,attr):
        if not self._loaded: self._preload("default")
        
        o = cls+":"+pcls+" "+attr
        
        #if not hasattr(self,'_count'):
        #    self._count = {}
        #if o not in self._count: self._count[o] = 0
        #self._count[o] += 1
        
        if o in self.cache: 
            return self.cache[o]
        
        v = self._get(cls+":"+pcls+" "+attr)
        if v: 
            self.cache[o] = v
            return v
        
        pcls = ""
        v = self._get(cls+":"+pcls+" "+attr)
        if v: 
            self.cache[o] = v
            return v
        
        cls = "default"
        v = self._get(cls+":"+pcls+" "+attr)
        if v: 
            self.cache[o] = v
            return v
        
        v = 0
        self.cache[o] = v
        return v
        
    def box(self,w,s):
        style = w.style
        style.cache(1)
        
        c = (0,0,0)
        if style.border_color != 0: c = style.border_color
        w,h = s.get_width(),s.get_height()
        
        s.fill(c,(0,0,w,style.border_top))
        s.fill(c,(0,h-style.border_bottom,w,style.border_bottom))
        s.fill(c,(0,0,style.border_left,h))
        s.fill(c,(w-style.border_right,0,style.border_right,h))
        
        style.cache(0)
        
    def getspacing(self,w):
        """return the top, right, bottom, left spacing around the widget"""
        if not hasattr(w,'_spacing'): #HACK: assume spacing doesn't change re pcls
            s = w.style
            s.cache(1)
            xt = s.margin_top+s.border_top+s.padding_top
            xr = s.padding_right+s.border_right+s.margin_right
            xb = s.padding_bottom+s.border_bottom+s.margin_bottom
            xl = s.margin_left+s.border_left+s.padding_left
            s.cache(0)
            w._spacing = xt,xr,xb,xl
        return w._spacing

        
    def resize(self,w,m):
        def func(width=None,height=None):
            #ww,hh = m(width,height)
            ow,oh = width,height
            
            
            s = w.style
            
            s.cache(1)
            pt,pr,pb,pl = s.padding_top,s.padding_right,s.padding_bottom,s.padding_left
            bt,br,bb,bl = s.border_top,s.border_right,s.border_bottom,s.border_left
            mt,mr,mb,ml = s.margin_top,s.margin_right,s.margin_bottom,s.margin_left
            s.cache(0)
            
            xt = pt+bt+mt
            xr = pr+br+mr
            xb = pb+bb+mb
            xl = pl+bl+ml
            ttw = xl+xr
            tth = xt+xb
            
            ww,hh = None,None
            if width != None: ww = width-ttw
            if height != None: hh = height-tth
            ww,hh = m(ww,hh)
            
            rect = pygame.Rect(0 + xl, 0 + xt, ww, hh)
            w._rect_content = rect #pygame.Rect(0 + xl, 0 + xt, width, height)
            #r = rect
        
            if width == None: width = ww
            if height == None: height = hh
            #if the widget hasn't respected the style.width,
            #style height, we'll add in the space for it...
            width = max(width-ttw,ww,w.style.width)
            height = max(height-tth,hh,w.style.height)
            
            #width = max(ww,w.style.width-tw)
            #height = max(hh,w.style.height-th)

            r = pygame.Rect(rect.x,rect.y,width,height)
            
            w._rect_padding = pygame.Rect(r.x-pl,r.y-pt,r.w+pl+pr,r.h+pt+pb)
            r = w._rect_padding
            w._rect_border = pygame.Rect(r.x-bl,r.y-bt,r.w+bl+br,r.h+bt+bb)
            r = w._rect_border
            w._rect_margin = pygame.Rect(r.x-ml,r.y-mt,r.w+ml+mr,r.h+mt+mb)
            
            #align it within it's zone of power.    
            dx = width-rect.w
            dy = height-rect.h
            #rect.x += (1)*dx/2
            #rect.y += (1)*dy/2
            rect.x += (w.style.align+1)*dx/2
            rect.y += (w.style.valign+1)*dy/2
            
            
            #print w,ow, w._rect_margin.w,  ttw
            return w._rect_margin.w,w._rect_margin.h
        return func
            
        
    def paint(self,w,m):
        def func(s):
            if hasattr(w,'background'):
                w.background.paint(mysubsurface(s,w._rect_border))
            self.box(w,mysubsurface(s,w._rect_border))
            r = m(mysubsurface(s,w._rect_content))
            return r
        return func
    
    def event(self,w,m):
        def func(e):
            rect = w._rect_content
            if e.type == MOUSEBUTTONUP or e.type == MOUSEBUTTONDOWN:
                sub = pygame.event.Event(e.type,{
                    'button':e.button,
                    'pos':(e.pos[0]-rect.x,e.pos[1]-rect.y)})
            elif e.type == CLICK:
                sub = pygame.event.Event(e.type,{
                    'button':e.button,
                    'pos':(e.pos[0]-rect.x,e.pos[1]-rect.y)})
            elif e.type == MOUSEMOTION:
                sub = pygame.event.Event(e.type,{
                    'buttons':e.buttons,
                    'pos':(e.pos[0]-rect.x,e.pos[1]-rect.y),
                    'rel':e.rel})
            else:
                sub = e
            r = m(sub)
            return r
        return func
    
    def update(self,w,m):
        def func(s):
            r = m(mysubsurface(s,w._rect_content))
            if type(r) == list:
                dx,dy = w._rect_content.topleft
                for rr in r:
                    rr.x,rr.y = rr.x+dx,rr.y+dy
            return r
        return func
        
    def open(self,w,m):
        def func(widget=None,x=None,y=None):
            if not hasattr(w,'_rect_content'): w.resize()
            rect = w._rect_content
            #print w.__class__.__name__, rect
            if x != None: x += rect.x
            if y != None: y += rect.y
            return m(widget,x,y)
        return func
        
    def decorate(self,w):
        w.paint = self.paint(w,w.paint)
        w.event = self.event(w,w.event)
        w.update = self.update(w,w.update)
        w.resize = self.resize(w,w.resize)
        w.open = self.open(w,w.open)
